/*
********************************************************************
* IBM Confidential                                                 *
*                                                                  *
* OCO Source Materials                                             *
*                                                                  *
* 5724-E76 and 5724-E77                                            *
*                                                                  *
* (C) Copyright IBM Corp. 2001, 2003                               *
*                                                                  *
* The source code for this program is not published or otherwise   *
* divested of its trade secrets, irrespective of what has been     *
* deposited with the U.S. Copyright Office.                        *
********************************************************************
*/

package com.ibm.wps.portletservice.credentialvault;

import java.util.Map;
import java.util.List;
import java.util.Iterator;
import java.util.Locale;

import org.apache.jetspeed.portlet.PortletRequest;
import org.apache.jetspeed.portlet.service.PortletService;
import org.apache.jetspeed.portlet.service.PortletServiceException;

import com.ibm.wps.portletservice.credentialvault.credentials.Credential;
import com.ibm.wps.services.credentialvault.VaultService;
import com.ibm.wps.sso.credentialvault.secrets.CredentialSecret;

import com.ibm.wps.util.ObjectID;

// JAAS classes
import javax.security.auth.Subject;

/**
 * Service that allows to retrieve credentials from the credential vault and to manage vault segments and slots.
 * 
 * @ibm-api
 * @since 4.1
 * @ibm-java2security
 **/
public interface CredentialVaultService extends PortletService
{
	static String PREDEFINED_SLOT_USER_JAAS_SUBJECT = VaultService.PREDEFINED_SLOT_USER_JAAS_SUBJECT;

    int SECRET_TYPE_UNDEFINED                     = CredentialSecret.TYPE_UNDEFINED;
    int SECRET_TYPE_USERID_STRING_PASSWORD_STRING = CredentialSecret.TYPE_USERID_STRING_PASSWORD_STRING;
    int SECRET_TYPE_BYTEARRAY                     = CredentialSecret.TYPE_BYTEARRAY;
    int SECRET_TYPE_JAVA_OBJECT                   = CredentialSecret.TYPE_JAVA_OBJECT;
    int SECRET_TYPE_NO_SECRET_DATA                = CredentialSecret.TYPE_NO_SECRET_DATA;
    int SECRET_TYPE_JAAS_SUBJECT                  = CredentialSecret.TYPE_JAAS_SUBJECT;
    

    /**
     *  Returns an Iterator of all Credential Types that are registered in the Credential Type Registry
     *
     *  @return   Iterator of String objects
     *            All Credential Types that are registered in the Credential Type Registry
     **/
    Iterator getCredentialTypes();
    

    /**
     *  Returns a Credential Slot's description for the specified locale.
     *
     *  @param   String
     *           The credential (slot) id.
     *
     *  @param   Locale
     *           The description locale. If set to <code>null</code>, the default locale will be used.
     *
     *  @return   String
     *            The credential slot description for the specified locale.
     *  @exception   PortletServiceException
     *               if the description could not been retrieved.
     **/
    String getSlotDescription(String slotId, Locale locale) throws PortletServiceException;
    
    
    /**
     *  Returns a Iterator of all credential slots that a portlet is authorized to use
     *
     *  @param   PortletRequest
     *           The portlet request is needed by the CredentialVault service in order to determine 
     *           information about the portlet.
     *
     *  @return   Iterator of CredentialSlotConfig
     *            All Credential Slots that a portlet is authorized to use
     *  @exception   PortletServiceException      
     *               if the list of slots could not been retrieved.
     **/
    Iterator getAccessibleSlots(PortletRequest request) throws PortletServiceException;
    

    /**
     *  Sets a credential's binary secret.
     *
     *  @param   String
     *           The credential (slot) id.
     *
     *  @param   byte[]
     *           The credential secret data in binary form.
     *
     *  @param   PortletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   PortletServiceException
     *               if the credential secret is not of the type binary or if the secret could not be set.
     **/
    void setCredentialSecretBinary(String slotId, byte[] secret, PortletRequest portletRequest) throws PortletServiceException;
    

    /**
     *  Sets a credential's user/password secret.
     *
     *  @param   String
     *           The credential (slot) id.
     *
     *  @param   String
     *           The credential's userId.
     *
     *  @param   char[]
     *           The credential's password.
     *
     *  @param   PortletRequest
     *           The portlet request is used to determine parameters like the portlet id and user id.
     *
     *  @exception   PortletServiceException
     *               if the credential secret is not of the type user/password or if the secret could not be set.
     **/
    void setCredentialSecretUserPassword(String slotId, String userId, char[] password, PortletRequest portletRequest) 
                                                throws PortletServiceException;
    

    /**
     * @deprecated
     * Deprecated: Use createSlot taking  com.ibm.portal.ObjectID segmentId as parameter.    
     *        
     * Creates a new Credential Slot.
     * This method should be used by "ordinary" portlets, but not by "admin portlets".
     *
     * @param String[64]
     *        Name of the resource
     * @param ObjectID
     *        Object id of the segment that this slot is created in
     * @param Map
     *        The slot descriptions as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters)
     *        Value: <code>String</code> (max length 255 characters)
     * @param int
     *        The secrtet type identifier
     * @param boolean
     *        Flag whether this credential may only be returned in form of an active credential object (<code>true</code>)
     *        or both as active and passive credential objects (<code>false</code>)
     * @param boolean
     *        Flag whether the credential secret is portlet secific (<code>true</code>) 
     *        or shared betwen all of a user's portlets (<code>false</code>).
     * @param PortletRequest
     *        The portlet request.
     * @return CredentialSlot
     *         The Credential Slot configuration
     **/
    CredentialSlotConfig createSlot( String resourceName, ObjectID segmentId,
                                            Map descriptions, Map keywords, 
                                            int secretType, boolean active, boolean portletPrivate,
                                            PortletRequest portletRequest)
                                            throws PortletServiceException;
    /**
     * Creates a new Credential Slot.
     * This method should be used by "ordinary" portlets, but not by "admin portlets".
     *
     * @param String[64]
     *        Name of the resource
     * @param ObjectID
     *        Object id of the segment that this slot is created in
     * @param Map
     *        The slot descriptions as a <code>Map</code>, keyed by their corresponding <code>Locale</code> objects
     *        Key:   <code>Locale</code> (max length 64 characters)
     *        Value: <code>String</code> (max length 255 characters)
     * @param int
     *        The secrtet type identifier
     * @param boolean
     *        Flag whether this credential may only be returned in form of an active credential object (<code>true</code>)
     *        or both as active and passive credential objects (<code>false</code>)
     * @param boolean
     *        Flag whether the credential secret is portlet secific (<code>true</code>) 
     *        or shared betwen all of a user's portlets (<code>false</code>).
     * @param PortletRequest
     *        The portlet request.
     * @return CredentialSlot
     *         The Credential Slot configuration
     **/
    CredentialSlotConfig createSlot( String resourceName, com.ibm.portal.ObjectID segmentId,
                                            Map descriptions, Map keywords, 
                                            int secretType, boolean active, boolean portletPrivate,
                                            PortletRequest portletRequest)
                                            throws PortletServiceException;

   /**
    * Deletes an existing Credential Slot.
    *
    *  @param   String
    *           The credential (slot) id.
    *  @exception   PortletServiceException
    *               if the credential slot could not be deleted.
    **/
    public void deleteSlot( String slotName ) throws PortletServiceException;

    /**
     * @deprecated
     * Deprecated: Use getDefaultUserVaultSegmentId returning  com.ibm.portal.ObjectID segmentId as parameter.    
     *            
     * Returns the ObjectID of the default user managed vault segment.
     * Currently there is only one user-managed segment, so this returns the ID of <i>the</i> user managed segment.
     *
     * @return ObjectID
     *         The ObjectID of the default user managed vault segment.
     **/
    ObjectID getDefaultUserVaultSegmentId() throws PortletServiceException;

    /**            
     * Returns the ObjectID of the default user managed vault segment.
     * Currently there is only one user-managed segment, so this returns the ID of <i>the</i> user managed segment.
     *
     * @return ObjectID
     *         The ObjectID of the default user managed vault segment.
     **/
    com.ibm.portal.ObjectID getDefaultUserVaultPortalSegmentId() throws PortletServiceException;

    /**
     * Returns a List of all Vault Segments.
     *
     * @return List of VaultSegmentConfig 
     *         All vault segments.
     **/
    List getAllVaultSegments() throws PortletServiceException;

    /**
     *  Returns a specific credential.
     * 
     *  @param   String
     *           The credential (slot) id.
     *
     *  @param   String
     *           The credential type as specified in the credential registry.
     *
     *  @param   Map
     *           The backend application specific configuration that is needed to initialize this credential. 
     *           This is one part of the credential configuration. The credential vault service will usually 
     *           add further information to this config from other sources: the user's secret from the actual credential store
     *           and the credential instace specific parameters from the portal's credential configuration.
     *
     *  @param   PortletRequest
     *           The portlet request is used to determine information about the portlet.
     *
     *  @return   Credential
     *            The credential.
     *
     *  @exception   PortletServiceException
     *               if the credential could not been retrieved - either for technical or secuity reasons.
     *
     *  @exception   CredentialNotSetException
     *               if the requested credential secret is not set (by the user or admin).
     **/
    Credential getCredential(String slotId, String type, Map config, PortletRequest request) 
                                    throws PortletServiceException, CredentialSecretNotSetException;

    /**
     *  Returns the user's JAAS subject. This is a special case of the getCredential call.
     * 
     *  @param   PortletRequest
     *           The portlet request is used to determine the user.
     *
     *  @return   Subject
     *            The user's JAAS subject.
     *
     *  @exception   PortletServiceException
     *               if the subject could not been retrieved.
     **/
    Subject getUserSubject(PortletRequest request) throws PortletServiceException;

}
